/*
* Copyright 2010,2013 EUROPEAN UNION
* Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
* the European Commission - subsequent versions of the EUPL (the "Licence");
* You may not use this work except in compliance with the Licence.
* You may obtain a copy of the Licence at:
*
* http://ec.europa.eu/idabc/eupl
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the Licence is distributed on an "AS IS" basis,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the Licence for the specific language governing permissions and
* limitations under the Licence.
*
* Date: 28/10/2013
* Authors: European Commission, Joint Research Centre
* Daniele Francioli, Emanuela Epure
* inspire-registry-info@jrc.ec.europa.eu
*
* This work was supported by the EU  Interoperability Solutions for
* European Public Administrations Programme (http://ec.europa.eu/isa)
* through Action 1.17: Re-usable INSPIRE Reference Platform 
* (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
*/

package eu.europa.ec.re3gistry.re3gistryexport.vargenerator;

import eu.europa.ec.re3gistry.re3gistryexport.manager.Configuration;
import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.servlet.ServletContext;
import org.apache.commons.io.FileUtils;

public class VarGenerator {

    private static final long serialVersionUID = 1L;
    private static final String KEY_URI_PREFIX = "var.prefix.uri";
    private static final String KEY_LANGUAGE_PREFIX = "var.prefix.language";
    private static final String KEY_TYPE_PREFIX = "var.prefix.type";
    private static final String KEY_EXPORT_MAINFOLDER_CUSTOM = "export.mainfolder.custom";
    private static final String KEY_EXPORT_MAINFOLDER_NAME = "export.mainfolder.name";
    private static final String KEY_EXPORT_FOLDERIDENTIFIER = "export.context.folderidentifier";
    private static final String KEY_VAR_EXCLUDE_FOLDER = "var.exclude.folder";
    private static String exportMainfolderCustom;
    private static String exportMainfolderName;
    private static String contextFolderIdentifier;
    private static String startFolder;
    private static String uriPrefix;
    private static String languagePrefix;
    private static String typePrefix;
    private static String lineSeparator;
    private static String folderSeparator;
    private static String excludeFolder;
    private List<File> folderList;
    private HashMap<String,String> excludeFolders; 

    public VarGenerator(ServletContext sc) throws Exception {
        /**
         * Getting properties
         */
        uriPrefix = Configuration.getInstance().getProperty(KEY_URI_PREFIX, "URI:");
        languagePrefix = Configuration.getInstance().getProperty(KEY_LANGUAGE_PREFIX, "Content-language:");
        typePrefix = Configuration.getInstance().getProperty(KEY_TYPE_PREFIX, "Content-type:");
        exportMainfolderCustom = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_CUSTOM, null);
        exportMainfolderName = Configuration.getInstance().getProperty(KEY_EXPORT_MAINFOLDER_NAME, "export");
        contextFolderIdentifier = Configuration.getInstance().getProperty(KEY_EXPORT_FOLDERIDENTIFIER, "WEB-INF");
        excludeFolder = Configuration.getInstance().getProperty(KEY_VAR_EXCLUDE_FOLDER, "");
        
        lineSeparator = System.getProperty("line.separator");
        folderSeparator = System.getProperty("file.separator");
        folderList = new ArrayList<File>();
        excludeFolders = new HashMap<String,String>();

        /**
         * Get the right directory for the job
         */
        if (exportMainfolderCustom != null && !exportMainfolderCustom.trim().equals("")) {
            startFolder = exportMainfolderCustom + folderSeparator + exportMainfolderName;
        } else if (sc != null) {
            String relativePath = sc.getRealPath(contextFolderIdentifier);
            startFolder = relativePath.replaceAll(contextFolderIdentifier, "") + exportMainfolderName;
        } else {
            throw new Exception("An error occurred while determining root ditectory");
        }
        
        /**
         * Get the exclude folder list
         */
        if (excludeFolder != null && !excludeFolder.trim().equals("")) {
            
            String [] temp = excludeFolder.split(",");
            for(int i=0; i<temp.length; i++){
                excludeFolders.put(temp[i],temp[i]);
            }
            
        }
    }

    public void generateVarFiles() throws Exception {
        if (startFolder != null && !startFolder.trim().equals("")) {
            File rootFolder = new File(startFolder);
            /**
             * Create (recursively) the list of all the directories to be processed
             */
            this.scanDirectories(rootFolder);

            /**
             * For each directory, create the .var file
             */
            for (int i = 0; i < folderList.size(); i++) {
                this.writeVar(folderList.get(i));
            }
        } else {
            throw new Exception("An error occurred while starting the var generation: VarGenerator has to be inizialized first!");
        }
    }

    private void writeVar(File folder) throws Exception {
        File[] fileList = folder.listFiles();
        String toVar = "";
        /**
         * For each file item in this folder
         */
        for (File file : fileList) {
            /**
             * Check if the file item is a file
             */
            if (file.isFile()) {
                /**
                 * Splitting the filename (pattern <filename>.<languagecode>.<extension>)
                 */
                String[] tmp = file.getName().split("\\.");
                if (tmp.length > 2) {
                    /**
                     * Getting the language code and the extension
                     */
                    String type = tmp[tmp.length - 1];
                    String language = tmp[tmp.length - 2];

                    /**
                     * Creating the var string for this file (language and
                     * extension information are taken from the properties file)
                     */
                    toVar += uriPrefix + " " + file.getName() + lineSeparator;
                    toVar += languagePrefix + " " + Configuration.getInstance().getProperty("content.language." + language) + lineSeparator;
                    toVar += typePrefix + " " + Configuration.getInstance().getProperty("content.type." + type) + lineSeparator;
                    toVar += lineSeparator;
                }
            }
        }
        /**
         * Writing the .var file
         */
        File var = new File(folder.getAbsolutePath() + folderSeparator + ".var");
        FileUtils.writeStringToFile(var, toVar);
    }

    private void scanDirectories(File inFolder) throws Exception {
        /**
         * Scanning the directories recursively in order to find subdirectories
         */
        File[] fl = inFolder.listFiles();
        for (File f : fl) {
            if (f.isDirectory()) {
                if(!excludeFolders.containsKey(f.getName())){
                    folderList.add(f);
                    scanDirectories(f);            
                }
            }
        }
    }
}