/*
 * Copyright 2010,2013 EUROPEAN UNION
 * Licensed under the EUPL, Version 1.1 or - as soon they will be approved by
 * the European Commission - subsequent versions of the EUPL (the "Licence");
 * You may not use this work except in compliance with the Licence.
 * You may obtain a copy of the Licence at:
 *
 * http://ec.europa.eu/idabc/eupl
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the Licence is distributed on an "AS IS" basis,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the Licence for the specific language governing permissions and
 * limitations under the Licence.
 *
 * Date: 28/10/2013
 * Authors: European Commission, Joint Research Centre
 * Daniele Francioli, Emanuela Epure
 * inspire-registry-info@jrc.ec.europa.eu
 *
 * This work was supported by the EU  Interoperability Solutions for
 * European Public Administrations Programme (http://ec.europa.eu/isa)
 * through Action 1.17: Re-usable INSPIRE Reference Platform 
 * (http://ec.europa.eu/isa/actions/01-trusted-information-exchange/1-17action_en.htm).
 */
package eu.europa.ec.re3gistry.re3gistryimport.manager;

import eu.europa.ec.re3gistry.exception.CsvException;
import eu.europa.ec.re3gistry.model.Customattribute;
import eu.europa.ec.re3gistry.model.Customattributevalue;
import eu.europa.ec.re3gistry.model.Item;
import eu.europa.ec.re3gistry.model.Itemclass;
import eu.europa.ec.re3gistry.model.Languagecode;
import eu.europa.ec.re3gistry.model.Localization;
import eu.europa.ec.re3gistry.model.Register;
import eu.europa.ec.re3gistry.model.Status;
import eu.europa.ec.re3gistry.re3gistryimport.constants.Constants;
import eu.europa.ec.re3gistry.util.StringUtils;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Date;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;
import javax.persistence.Query;
import org.apache.log4j.Logger;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import java.io.File;
import java.io.FileInputStream;
import java.util.ArrayList;

public class ImportMgr {

    private static final String DESCRIPTOR_FILENAME = "descriptor.xml";
    //Required variables
    private static Logger log = Logger.getLogger(ImportMgr.class);
    private static EntityManagerFactory factory;
    private String csvDelimiter;
    private String persistenceUnitName;
    private String customAttributePrefix;
    private String customAttribute_properties_container_open;
    private String customAttribute_properties_container_close;
    private String customAttribute_properties_container_separator;
    private String defaultStatudUuid;
    private String defaultNoLabel;
    private String replaceRegex;
    private int standardColumnNumber;
    //Constant key (used to retrieve properties)
    private static final String KEY_PERSISTENCE_UNIT_NAME = "persistence.unit.name";
    private static final String KEY_CSV_DELIMITER = "csv.delimiter";
    private static final String KEY_CSV_STANDARD_COLUMN = "csv.standard_field.number";
    private static final String KEY_CSV_CUSTOMATTRIBUTE_PREFIX = "csv.customattribute.prefix";
    private static final String KEY_CSV_CUSTOMATTRIBUTE_PROPERTIESCONTAINER_OPEN = "csv.customattribute.propertiescontainer.open";
    private static final String KEY_CSV_CUSTOMATTRIBUTE_PROPERTIESCONTAINER_CLOSE = "csv.customattribute.propertiescontainer.close";
    private static final String KEY_CSV_CUSTOMATTRIBUTE_PROPERTIESCONTAINER_SEPARATOR = "csv.customattribute.propertiescontainer.separator";
    private static final String KEY_CSV_NULL_LABEL = "csv.label.null.value";
    //Constant key related to a specific import
    private static final String KEY_IMPORT_STATUS = "import.default.status.uuid";
    //Replace string regex key
    private static final String KEY_REPLACE_REGEX = "import.regex.code.replace";

    public ImportMgr() {
        this.csvDelimiter = Configuration.getInstance().getProperty(KEY_CSV_DELIMITER, ";");
        this.standardColumnNumber = Configuration.getInstance().getProperty(KEY_CSV_STANDARD_COLUMN, 8);
        this.customAttributePrefix = Configuration.getInstance().getProperty(KEY_CSV_CUSTOMATTRIBUTE_PREFIX, "*");
        this.customAttribute_properties_container_open = Configuration.getInstance().getProperty(KEY_CSV_CUSTOMATTRIBUTE_PROPERTIESCONTAINER_OPEN, "[");
        this.customAttribute_properties_container_close = Configuration.getInstance().getProperty(KEY_CSV_CUSTOMATTRIBUTE_PROPERTIESCONTAINER_CLOSE, "]");
        this.customAttribute_properties_container_separator = Configuration.getInstance().getProperty(KEY_CSV_CUSTOMATTRIBUTE_PROPERTIESCONTAINER_SEPARATOR, ",");
        this.persistenceUnitName = Configuration.getInstance().getProperty(KEY_PERSISTENCE_UNIT_NAME, "Re3gistry");
        this.defaultStatudUuid = Configuration.getInstance().getProperty(KEY_IMPORT_STATUS, "1");
        this.defaultNoLabel = Configuration.getInstance().getProperty(KEY_CSV_NULL_LABEL, "No label");
        this.replaceRegex = Configuration.getInstance().getProperty(KEY_REPLACE_REGEX, "");
    }

    private Customattribute[] checkCsvHeader(String s, Itemclass itemclass) throws CsvException, Exception {
        String[] elements = s.split(this.csvDelimiter);
        int standardFieldCount = 0;
        int customAttributeCount = 0;

        //Count if the standard csv fields are available (the first fields are the 
        //standard fields in the CSV structure. They have to be provided even if
        //they are null)
        //Header example:
        //code|parentcode|collectioncode|predecessorcode|successorcode|label|description|definition|*customattributename[t,f,f,f]|...
        for (String element : elements) {
            element = element.trim();
            if (element != null && !element.equals("") && !element.startsWith(this.customAttributePrefix)) {
                standardFieldCount++;
            } else if (element != null && !element.equals("") && element.startsWith(this.customAttributePrefix)) {
                customAttributeCount++;
            }
        }

        //Report an error if the standard fields number are not as configured in the properties file
        if (standardFieldCount != this.standardColumnNumber) {
            throw new CsvException("Warning: please check the csv file format");
        }

        //Handling the custom attributes: parsing the custom attribute header and
        //creating the custom attribute descriptor
        String ca_string = null;
        String ca_name = null;
        String ca_property = null;
        Customattribute[] cas = null;
        String[] ca_properties = null;
        int cnt = 0;

        //If the previous parser found some custom attribute
        if (customAttributeCount > 0) {
            cas = new Customattribute[customAttributeCount];

            //Parsing and setting the custom attribute descriptor and properties
            for (int i = standardFieldCount; i < elements.length; i++) {
                ca_string = elements[i].trim();
                ca_name = null;
                ca_property = null;

                //Splitting custom attribute name and properties
                if (ca_string != null && !ca_string.equals("") && ca_string.startsWith(this.customAttributePrefix)) {
                    ca_name = ca_string.substring(1, ca_string.indexOf(this.customAttribute_properties_container_open));
                    ca_property = ca_string.substring(ca_string.indexOf(this.customAttribute_properties_container_open) + 1, ca_string.indexOf(this.customAttribute_properties_container_close));
                }

                if (ca_name != null && !ca_name.trim().equals("") && ca_property != null && !ca_property.trim().equals("")) {
                    ca_properties = ca_property.split(this.customAttribute_properties_container_separator);
                    if (ca_properties != null && ca_properties.length == 4) {

                        //Creating the custom attribute descriptor element
                        Customattribute ca = new Customattribute();

                        //Creating uuid (custom attribute descriptor name + itemclass uri name, Customattributevalue class)
                        String cauuid = StringUtils.createUUID(ca_name + itemclass.getUriName(), Customattribute.class);

                        //Setting custom attribute descriptor properties
                        ca.setUuid(cauuid);
                        ca.setName(ca_name);
                        ca.setRequired(ca_properties[0].equals("t") ? true : false);
                        ca.setMultivalue(ca_properties[1].equals("t") ? true : false);
                        ca.setCoded(ca_properties[2].equals("t") ? true : false);
                        ca.setIsforeignkey(ca_properties[3].equals("t") ? true : false);
                        ca.setDatecreation(new Date());
                        ca.setItemclass(itemclass);

                        cas[cnt] = ca;
                        cnt++;
                    } else {
                        log.info("Error while creating a custom attribute from csv header - field n. " + (i + 1));
                        cas[cnt] = null;
                        cnt++;
                    }
                } else {
                    log.info("Error while creating a custom attribute from csv header - field n. " + (i + 1));
                    cas[cnt] = null;
                    cnt++;
                }
            }
        }
        return cas;
    }

    public int startImport(String tempFolder) throws Exception {

        //Checking folder
        File folder = new File(tempFolder);
        if (!folder.exists()) {
            throw new Exception("The temp folder was not found.");
        }

        File descriptor = new File(tempFolder + File.separator + DESCRIPTOR_FILENAME);

        if (!descriptor.isFile()) {
            throw new Exception("No import descriptor found in the zip file");
        }

        //Parsing the xml descriptor file
        DocumentBuilderFactory dbFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder dBuilder = dbFactory.newDocumentBuilder();
        Document doc = dBuilder.parse(descriptor);

        NodeList nl = doc.getElementsByTagName("registers").item(0).getChildNodes();

        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            if (n.getNodeType() == Node.ELEMENT_NODE) {

                //Getting informations for each registers                
                Element el = (Element) n;
                String registerID = el.getAttribute("id");

                Node nitemclass = el.getElementsByTagName("itemclass").item(0);
                Node nfolder = el.getElementsByTagName("folder").item(0);

                String itemclassID = ((Element) nitemclass).getAttribute("id");
                String folderName = ((Element) nfolder).getAttribute("name");

                NodeList nll = el.getElementsByTagName("languages").item(0).getChildNodes();
                for (int j = 0; j < nll.getLength(); j++) {
                    Node l = nll.item(j);
                    if (l.getNodeType() == Node.ELEMENT_NODE) {
                        //Processing each language files
                        Element ll = (Element) l;
                        String languageCode = ll.getAttribute("code");
                        String master = ll.getAttribute("master");
                        String collectionUUID = ll.getAttribute("itemClassCollectionId");
                        int action;

                        log.info(">>Now processing register: " + registerID + ", itemclass: " + itemclassID + ", language: " + languageCode);

                        if (master.equals("true")) {
                            log.info("This is a master import;");
                            action = Constants.ACTION_TYPE_ITEMIMPORT;
                        } else {
                            log.info("This is a language import: " + languageCode);
                            action = Constants.ACTION_TYPE_LANGUAGEIMPORT;
                        }

                        File f = new File(tempFolder + File.separator + folderName + File.separator + languageCode + ".csv");

                        this.dataImport(registerID, itemclassID, collectionUUID, languageCode, action, f);

                        if (master.equals("true")) {
                            log.info("This is a master language import: " + languageCode);
                            this.dataImport(registerID, itemclassID, collectionUUID, languageCode, Constants.ACTION_TYPE_LANGUAGEIMPORT, f);
                        }

                    }

                }
            }
        }

        return 0;
    }

    public int dataImport(String registerUUID, String itemclassUUID, String itemclassCollectionUUID, String languageCode, int action, File inFile) {

        //Istantiating variables
        int totalImportedRow = 0;
        int line = 1;
        EntityManager em = null;
        String text = "";
        String[] sp = null;
        String[] ca_values = null;
        Customattribute[] ca = null;
        ArrayList parentList = new ArrayList();

        try {
            //Istantiating persistence manager
            factory = Persistence.createEntityManagerFactory(this.persistenceUnitName);
            em = factory.createEntityManager();

            //Getting register object
            Query q = em.createNamedQuery("Register.findByUuid");
            q.setParameter("uuid", registerUUID);
            Register register = (Register) q.getSingleResult();

            //Getting itemclass object
            q = em.createNamedQuery("Itemclass.findByUuid");
            q.setParameter("uuid", itemclassUUID);
            Itemclass itemclass = (Itemclass) q.getSingleResult();

            //Getting itemclass collection object
            Itemclass itemclassCollection = null;
            if (itemclassCollectionUUID != null && !itemclassCollectionUUID.trim().equals("")) {
                q = em.createNamedQuery("Itemclass.findByUuid");
                q.setParameter("uuid", itemclassCollectionUUID);
                itemclassCollection = (Itemclass) q.getSingleResult();
            }

            //Getting languagecode object
            q = em.createNamedQuery("Languagecode.findByIsocode");
            q.setParameter("isocode", languageCode);
            Languagecode languagecode = (Languagecode) q.getSingleResult();

            //Checking if the required parameter are not null
            if (languagecode != null && itemclass != null && register != null) {

                //The item import can be done only if no data is found in the DB
                //for the current register & itemclass (First import)
                q = em.createNamedQuery("Item.findByRegisterItemclass");
                q.setParameter("register", register);
                q.setParameter("itemclass", itemclass);
                if ((q.getResultList().size() <= 0 && action == Constants.ACTION_TYPE_ITEMIMPORT) || (action == Constants.ACTION_TYPE_LANGUAGEIMPORT)) {

                    //Getting uploaded file & checking it
                    if(!inFile.exists()) throw new IOException(); 
                    
                    InputStream filecontent = new FileInputStream(inFile);
                    if (filecontent != null) {
                        BufferedReader in = new BufferedReader(new InputStreamReader(filecontent, "UTF-8"));
                        log.info("Starting parser;");

                        //Getting transaction
                        em.getTransaction().begin();

                        while ((text = in.readLine()) != null) {

                            //Checking csv header (and eventually creating custom
                            //attribute descriptor)
                            if (line == 1) {
                                ca = this.checkCsvHeader(text, itemclass);
                                line++;
                                continue;
                            }

                            //Splitting row for the import
                            sp = text.split(this.csvDelimiter, -1);

                            //Checking if there is the right number of standard column
                            //(as setted in the properties file)
                            if (!(sp.length >= this.standardColumnNumber)) {
                                log.info("Warning: row " + line + ", doesn't have the right required fields number. This is row will be skipped;");
                                line++;
                                continue;
                            }

                            //Finding, normalizing (if required) and setting the elemenst as specified in the csv 
                            //format (first elements are standard in the csv: they have to be provided even if they are null). 
                            //Fields & order:
                            //code|parentcode|collectioncode|predecessorcode|successorcode|label|definition|description

                            //csv position 0: code - Required in order to import the row
                            String uri_name = StringUtils.normalize(sp[0]).replaceAll("[\u0000-\u001f]", "");
                            uri_name = uri_name.replaceAll(this.replaceRegex, "");
                            if (uri_name == null || uri_name.equals("")) {
                                log.info("Warning: row " + line + ", missing field code. This is row will be skipped;");
                                line++;
                                continue;
                            }

                            if (uri_name.equals("publicAdministrationOffice")) {
                                uri_name = uri_name;
                            }

                            //Getting other values from the csv

                            //Calculating uuid from code (item uri_name + item class uri_name, itemclass class)
                            String ituuid = StringUtils.createUUID(uri_name + itemclass.getUriName(), Itemclass.class);

                            //csv position 1: parentcode
                            String parentcode = StringUtils.normalize(sp[1]).replaceAll("[\u0000-\u001f]", "");
                            //csv position 2: collectioncode
                            String collectioncode = StringUtils.normalize(sp[2]).replaceAll("[\u0000-\u001f]", "");
                            //csv position 3: predecessorcode
                            String predecessorcode = StringUtils.normalize(sp[3]).replaceAll("[\u0000-\u001f]", "");
                            //csv position 4: successorcode
                            String successorcode = StringUtils.normalize(sp[4]).replaceAll("[\u0000-\u001f]", "");
                            //csv position 5: label
                            String label = sp[5].trim().replaceAll("[\u0000-\u001f]", "");
                            //csv position 6: definition
                            String definition = sp[6].trim().replaceAll("[\u0000-\u001f]", "");
                            //csv position 7: description
                            String description = sp[7].trim().replaceAll("[\u0000-\u001f]", "");
                            //csv position 8: status
                            String statusuuid = sp[8].trim().replaceAll("[\u0000-\u001f]", "");

                            //Getting status object
                            q = em.createNamedQuery("Status.findByUuid");
                            q.setParameter("uuid", statusuuid);
                            Status status = null;
                            try {
                                status = (Status) q.getSingleResult();
                            } catch (Exception e) {
                                q = em.createNamedQuery("Status.findByUuid");
                                q.setParameter("uuid", defaultStatudUuid);
                                status = (Status) q.getSingleResult();
                                log.info("Warning: row " + line + ", stasus code cannot be found. The default status is being setted;");
                            }

                            //Create new item
                            Item it = new Item();
                            //Create new localization
                            Localization lo = new Localization();

                            //Setting required fields
                            it.setUuid(ituuid);
                            it.setRegister(register);
                            it.setItemclass(itemclass);
                            it.setUriName(uri_name);
                            it.setStatus(status);

                            Date d = new Date();
                            it.setDatecreation(d);
                            it.setDatetimeaddition(d);

                            //Setting optional fields
                            if (parentcode != null && !parentcode.equals("")) {
                                /*q = em.createNamedQuery("Item.findByUuid");
                                 q.setParameter("uuid", StringUtils.createUUID(parentcode + itemclass.getUriName(), Itemclass.class));
                                 try {
                                 it.setParent((Item) q.getSingleResult());
                                 } catch (Exception e) {
                                 log.info("Warning, row " + line + ", no reference found for parent code: " + parentcode);
                                 }*/
                                Object o[] = new Object[2];
                                o[0] = it;
                                o[1] = parentcode;
                                parentList.add(o);
                            }

                            //Collection code
                            if (collectioncode != null && !collectioncode.equals("")) {
                                q = em.createNamedQuery("Item.findByUuid");
                                q.setParameter("uuid", StringUtils.createUUID(collectioncode + itemclassCollection.getUriName(), Itemclass.class));
                                try {
                                    it.setCollection((Item) q.getSingleResult());
                                } catch (Exception e) {
                                    //If there is a reference to the collectioncode in the CSV but the system
                                    //can't find the element in the db, the row is skipped
                                    //because the collection item cannot have the collection master setted
                                    log.info("Warning, row " + line + ", no reference found for collection code: " + collectioncode + ". This row will be skipped;");
                                    line++;
                                    continue;
                                }
                            } else {
                                if (itemclass.getOrdernumber() > 0) {
                                    //If the item class order number is greater than zero, that it need
                                    //the collection code in order to set the collection master.
                                    //If the CSV row has no reference for this element, the row is skipped.
                                    log.info("Warning, row " + line + ", no collection code found in the CSV. This row will be skipped;");
                                    line++;
                                    continue;
                                }
                            }

                            if (predecessorcode != null && !predecessorcode.equals("")) {
                                q = em.createNamedQuery("Item.findByUuid");
                                q.setParameter("uuid", StringUtils.createUUID(predecessorcode + itemclass.getUriName(), Itemclass.class));
                                try {
                                    it.setPredecessor((Item) q.getSingleResult());
                                } catch (Exception e) {
                                    log.info("Warning, row " + line + ", no reference found for predecessor code: " + predecessorcode);
                                }
                            }

                            if (successorcode != null && !successorcode.equals("")) {
                                q = em.createNamedQuery("Item.findByUuid");
                                q.setParameter("uuid", StringUtils.createUUID(successorcode + itemclass.getUriName(), Itemclass.class));
                                try {
                                    it.setSuccessor((Item) q.getSingleResult());
                                } catch (Exception e) {
                                    log.info("Warning, row " + line + ", no reference found for successor code: " + successorcode);
                                }
                            }

                            //If the item is a collection item, the uuid generation method
                            //is changed (item uri_name + collection uri_name + itemclass uri_name, Itemclass class)
                            if (it.getCollection() != null && itemclass.getOrdernumber() > 0) {
                                ituuid = StringUtils.createUUID(uri_name + it.getCollection().getUriName() + itemclass.getUriName(), Itemclass.class);
                                it.setUuid(ituuid);
                            }

                            //Checking import action (item or language)
                            if (action == Constants.ACTION_TYPE_ITEMIMPORT) {
                                //Checking if this item is already in the db
                                q = em.createNamedQuery("Item.findByUuid");
                                q.setParameter("uuid", ituuid);
                                if (q.getResultList().size() > 0) {
                                    log.info("Warning: row " + line + ", the item is already in the DB. This row will be skipped;");
                                    line++;
                                    continue;
                                }
                                //Saving this item
                                em.persist(it);

                            } else if (action == Constants.ACTION_TYPE_LANGUAGEIMPORT) {

                                //Check if a label is already in the db for this item & language
                                q = em.createNamedQuery("Localization.findByItemLanguagecode");
                                q.setParameter("item", it);
                                q.setParameter("language", languagecode);
                                try {
                                    Localization tmp = (Localization) q.getSingleResult();
                                    log.info("Warning: row " + line + ", a localization for item " + it.getUriName() + " and language " + languagecode.getLabel() + " is already in the DB. This localization will be skipped;");
                                    line++;
                                    continue;
                                } catch (Exception e) {
                                }

                                //Creating localization uuid
                                String louuid = StringUtils.createUUID(ituuid + System.currentTimeMillis(), Localization.class);

                                //Setting fields
                                lo.setUuid(louuid);

                                //Checking if the item referenced by this localization is in the DB
                                q = em.createNamedQuery("Item.findByUuid");
                                q.setParameter("uuid", it.getUuid());
                                try {
                                    it = (Item) q.getSingleResult();
                                } catch (Exception e) {
                                    log.info("Warning: row " + line + ", no reference found in the DB for item " + it.getUuid() + ". This localization will be skipped;");
                                    line++;
                                    continue;
                                }
                                lo.setItem(it);

                                //Checking label: if the label is null the uri name is used as label 
                                if (label != null && !label.trim().equals("")) {
                                    lo.setLabel(label);
                                } else {
                                    lo.setLabel(uri_name);
                                    log.info("Warning: row " + line + ", the item has no label. The uri name is setted as label");
                                }

                                lo.setDefinition((definition != null && !definition.trim().equals("")) ? definition : null);
                                lo.setDescription((description != null && !description.trim().equals("")) ? description : null);
                                lo.setDatecreation(new Date());
                                lo.setLanguage(languagecode);
                                //Saving localization
                                em.persist(lo);

                                //Parsing Custom attributes value
                                ca_values = new String[sp.length - this.standardColumnNumber];
                                int i, j = 0;
                                for (i = this.standardColumnNumber, j = 0; i < sp.length; i++, j++) {
                                    ca_values[j] = sp[i];
                                }
                            }

                            //Handling custom attributes & custom attributes values
                            if (ca != null) {
                                int cavtot = 0;
                                for (int i = 0; i < ca.length; i++) {

                                    //If it is a language import, the custom attribute descriptor has to be the same number of the custom attribute number
                                    if (action == Constants.ACTION_TYPE_LANGUAGEIMPORT && (ca.length != ca_values.length)) {
                                        log.info("Warning: row " + line + ", found different custom attribute descriptor numnber and custom attribute values. Custom attributes at this row will be skipped;");
                                        break;
                                    }

                                    //Getting custom attribute descriptor 
                                    //(previously parsed from the csv header)
                                    Customattribute c = ca[i];

                                    //Checking if this custom attribute descriptor is already in the DB
                                    q = em.createNamedQuery("Customattribute.findByUuid");
                                    q.setParameter("uuid", c.getUuid());
                                    try {
                                        c = (Customattribute) q.getSingleResult();
                                    } catch (Exception e) {
                                        if (action == Constants.ACTION_TYPE_ITEMIMPORT) {
                                            //If the customattribute descriptor is not already in the db
                                            //and this is an item import, it is saved
                                            em.persist(c);
                                        } else if (action == Constants.ACTION_TYPE_LANGUAGEIMPORT) {
                                            log.info("Warning: row " + line + ", cannot find a custom attribute descriptor for language import at this row; This attribute will be skipped;");
                                            line++;
                                            continue;
                                        }
                                    }

                                    //Creating custom attribute value for this row
                                    Customattributevalue cav = new Customattributevalue();

                                    //Creating uuid (custom attribute descriptor name + item uri name + item class uri name, Customattributevalue class)
                                    String cavuuid = StringUtils.createUUID(c.getName() + it.getUriName() + it.getItemclass().getUriName(), Customattributevalue.class);

                                    //Setting values
                                    cav.setUuid(cavuuid);
                                    cav.setCustomattribute(c);
                                    cav.setItem(it);
                                    cav.setDatecreation(new Date());

                                    //Checking if the custom attribute value descriptor
                                    //is not already in ther db
                                    q = em.createNamedQuery("Customattributevalue.findByUuid");
                                    q.setParameter("uuid", cav.getUuid());
                                    try {
                                        cav = (Customattributevalue) q.getSingleResult();
                                    } catch (Exception e) {
                                        if (action == Constants.ACTION_TYPE_ITEMIMPORT) {
                                            //If the customattribute value descriptor is not already in the db
                                            //and this is an item import, it is saved
                                            em.persist(cav);
                                            cavtot++;
                                            //log.info(line + " saving CA_VALUE_DESCRIPTR " + cavtot);
                                        } else if (action == Constants.ACTION_TYPE_LANGUAGEIMPORT) {
                                            log.info("Warning: row " + line + ", cannot find a custom attribute value descriptor for language import at this row. This attribute will be skipped;");
                                            line++;
                                            continue;
                                        }
                                    }

                                    //If the import is a language import, start
                                    //the custom attribute values import
                                    if (action == Constants.ACTION_TYPE_LANGUAGEIMPORT) {
                                        lo = new Localization();

                                        //Creating localization uuid
                                        String louuid = StringUtils.createUUID(cavuuid + System.currentTimeMillis(), Localization.class);

                                        //Setting fields
                                        lo.setUuid(louuid);
                                        lo.setCustomattributevalue(cav);
                                        //Setting label
                                        String lbl = ca_values[i];
                                        if (lbl != null && !lbl.trim().equals("")) {

                                            lo.setLabel(lbl);

                                        } else {
                                            lo.setLabel(defaultNoLabel);
                                            log.info("Warnind: row " + line + ", custom attribute has no label. A standard label is setted: " + defaultNoLabel);
                                        }
                                        lo.setDatecreation(new Date());
                                        lo.setLanguage(languagecode);

                                        em.persist(lo);
                                    }
                                }
                            }
                            totalImportedRow++;
                            line++;
                        }
                    } else {
                        log.info("Warning: no file provided;");
                    }
                } else {
                    log.info("Warning: this item import has already been performed. To re-import this file you have to delete all the data previously imported;");
                }
            } else {
                log.info("Warning: required parameter missing; the import cannot be performed;");
            }

            //Commit transaction and close db connection
            em.getTransaction().commit();

            //Setting parent code
            if (action == Constants.ACTION_TYPE_ITEMIMPORT) {
                em.getTransaction().begin();
                if (parentList != null && parentList.size() > 0) {
                    for (int i = 0; i < parentList.size(); i++) {
                        Object o[] = (Object[]) parentList.get(i);
                        Item it = (Item) o[0];
                        String parentUriName = (String) o[1];
                        q = em.createNamedQuery("Item.findByUuid");

                        String tmpUuid = null;

                        if (it.getCollection() != null && itemclass.getOrdernumber() > 0) {
                            tmpUuid = StringUtils.createUUID(parentUriName + it.getCollection().getUriName() + itemclass.getUriName(), Itemclass.class);
                        } else {
                            tmpUuid = StringUtils.createUUID(parentUriName + itemclass.getUriName(), Itemclass.class);
                        }

                        q.setParameter("uuid", tmpUuid);
                        try {
                            log.info("Setting parent for item: " + it.getUriName() + ";");
                            it.setParent((Item) q.getSingleResult());
                            em.persist(it);
                        } catch (Exception e) {
                            log.info("Warning, no reference found for parent code: " + parentUriName);
                        }
                    }
                }
                em.getTransaction().commit(); 
            }

            //Closing transaction
            em.close();


            if (totalImportedRow == 0) {
                log.info("Warning: no row imported;");
            } else {
                log.info("Total imported rows " + totalImportedRow);
            }

        } catch (IOException e) {
            log.error("There was a IO error while importing the uploaded file;", e);
        } catch (CsvException e) {
            log.error("There was an error while parsing the csv file;", e);
        } catch (Exception e) {
            log.error("There was a generic error while executing the import;", e);
        }
        return totalImportedRow;
    }
}